#!/usr/bin/env python3

import pytest
from MCP2221 import MCP2221


def testClockOut():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(1, MCP2221.TYPE.CLOCK_OUT)
    mcp2221.SetClockOutput(MCP2221.DUTY.CYCLE_50, MCP2221.CLOCK.DIV_3MHZ)
    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    duty = (buf[5] >> 3) & 0b11
    clock = buf[5] & 0b111

    assert (duty, clock) == (
        MCP2221.DUTY.CYCLE_50.value, MCP2221.CLOCK.DIV_3MHZ.value)


def testInvalidClockPin():
    mcp2221 = MCP2221.MCP2221()

    with pytest.raises(TypeError):
        mcp2221.InitGP(0, MCP2221.TYPE.CLOCK_OUT)


def testInvalidClockDuty():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(1, MCP2221.TYPE.CLOCK_OUT)

    with pytest.raises(TypeError):
        mcp2221.SetClockOutput(123, MCP2221.CLOCK.DIV_3MHZ)


def testInvalidClockDiv():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(1, MCP2221.TYPE.CLOCK_OUT)

    with pytest.raises(TypeError):
        mcp2221.SetClockOutput(MCP2221.DUTY.CYCLE_25, 123)



#!/usr/bin/env python3

import pytest
from MCP2221 import MCP2221


@pytest.mark.parametrize("pin", [0, 1, 2, 3])
def testInitOutput(pin):
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(pin, MCP2221.TYPE.OUTPUT)

    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    gp_type = buf[pin + 22] & 0b111
    direction = ((buf[pin + 22] >> 3) & 0b1)

    assert (gp_type, direction) == (0, 0)


@pytest.mark.parametrize("pin", [0, 1, 2, 3])
def testRead(pin):
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(pin, MCP2221.TYPE.OUTPUT)
    mcp2221.WriteGP(pin, 1)
    state = mcp2221.ReadGP(pin)

    assert state == 1


def testIOAll():
    mcp2221 = MCP2221.MCP2221()
    for i in range(4):
        mcp2221.InitGP(i, MCP2221.TYPE.OUTPUT)

    mcp2221.WriteAllGP(1, 1, 1, 1)
    state = mcp2221.ReadAllGP()

    assert state == [1, 1, 1, 1]


@pytest.mark.parametrize("pin", [0, 1, 2, 3])
def testInitInput(pin):
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(pin, MCP2221.TYPE.INPUT)

    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    gp_type = buf[pin + 22] & 0b111
    direction = ((buf[pin + 22] >> 3) & 0b1)

    assert (gp_type, direction) == (0, 1)


@pytest.mark.parametrize("pin,type,value", [
    (0, MCP2221.TYPE.SSPND, 0b001),
    (0, MCP2221.TYPE.LED_RX, 0b010),
    (1, MCP2221.TYPE.LED_TX, 0b011),
    (2, MCP2221.TYPE.USBCFG, 0b001),
    (3, MCP2221.TYPE.LED_I2C, 0b001)])
def testValidType(pin, type, value):
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(pin, type)

    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    gp_type = buf[pin + 22] & 0b111

    assert gp_type == value


@pytest.mark.parametrize("pin,type", [
    (0, MCP2221.TYPE.LED_I2C),
    (1, MCP2221.TYPE.LED_RX),
    (2, MCP2221.TYPE.SSPND),
    (3, MCP2221.TYPE.CLOCK_OUT)])
def testInvalidType(pin, type):
    mcp2221 = MCP2221.MCP2221()

    with pytest.raises(TypeError):
        mcp2221.InitGP(pin, type)


def testInitInvalidPinNumber():
    mcp2221 = MCP2221.MCP2221()

    with pytest.raises(ValueError):
        mcp2221.InitGP(4, MCP2221.TYPE.INPUT)


def testInvalidReadPin():
    mcp2221 = MCP2221.MCP2221()

    with pytest.raises(ValueError):
        mcp2221.ReadGP(-1)


def testInvalidWritePin():
    mcp2221 = MCP2221.MCP2221()

    with pytest.raises(ValueError):
        mcp2221.WriteGP(4, 1)


def testCombinated():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(0, MCP2221.TYPE.INPUT)
    mcp2221.InitGP(1, MCP2221.TYPE.OUTPUT)
    mcp2221.InitGP(2, MCP2221.TYPE.ADC)
    mcp2221.InitGP(3, MCP2221.TYPE.DAC)

    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    gp0 = buf[22] & 0b111
    gp0_dir = ((buf[22] >> 3) & 0b1)
    gp1 = buf[23] & 0b111
    gp1_dir = ((buf[23] >> 3) & 0b1)
    gp2 = buf[24] & 0b111
    gp3 = buf[25] & 0b111

    assert (gp0, gp0_dir, gp1, gp1_dir, gp2, gp3) == (
        0b000, 1, 0b000, 0, 0b010, 0b011)



#!/usr/bin/env python3

import pytest
from MCP2221 import MCP2221


def testInitInterrupt():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(1, MCP2221.TYPE.INTERRUPT)

    with pytest.raises(Exception):
        mcp2221.SetInterruptDetection()


def testInterruptPin():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(1, MCP2221.TYPE.INTERRUPT)

    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    gp_type = buf[23] & 0b111

    assert gp_type == 0b100



#!/usr/bin/env python3

import pytest
from MCP2221 import MCP2221


@pytest.mark.parametrize("pin", [1, 2, 3])
def testInitADC(pin):
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(pin, MCP2221.TYPE.ADC)

    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    gp_type = buf[pin + 22] & 0b111

    assert gp_type == 0b010


def testVrefVDD():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.SetADCVoltageReference(MCP2221.VRM.VDD)

    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    vref = (buf[7] >> 3) & 0b11
    vrm = (buf[7] >> 2) & 0b1

    assert (vref, vrm) == (MCP2221.VRM.VDD.value, 0)


def testVrefVRM():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.SetADCVoltageReference(MCP2221.VRM.REF_2_048V)

    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    vref = (buf[7] >> 3) & 0b11
    vrm = (buf[7] >> 2) & 0b1

    assert (vref, vrm) == (MCP2221.VRM.REF_2_048V.value, 1)


def testInvalidVref():
    mcp2221 = MCP2221.MCP2221()

    with pytest.raises(TypeError):
        mcp2221.SetADCVoltageReference(123)


def testReadAllADC():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(1, MCP2221.TYPE.ADC)
    mcp2221.InitGP(2, MCP2221.TYPE.ADC)
    mcp2221.InitGP(3, MCP2221.TYPE.ADC)

    adc = mcp2221.ReadAllADC()

    assert adc is not None


@pytest.mark.parametrize("pin", [1, 2, 3])
def testReadADC(pin):
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(pin, MCP2221.TYPE.ADC)

    adc = mcp2221.ReadADC(pin)

    assert 0 < adc < 1024


def testInvalidMaxADC():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(2, MCP2221.TYPE.ADC)

    with pytest.raises(ValueError):
        mcp2221.ReadADC(4)


def testInvalidMinADC():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(2, MCP2221.TYPE.ADC)

    with pytest.raises(ValueError):
        mcp2221.ReadADC(0)



#!/usr/bin/env python3

import pytest
from MCP2221 import MCP2221


def testDeviceInfo():
    mcp2221 = MCP2221.MCP2221()
    info = mcp2221.GetDeviceInfo()

    m = "manufacturer" in info
    p = "product" in info
    s = "serial" in info

    assert (m, p, s) == (True, True, True)


def testWriteReadFlash():
    mcp2221 = MCP2221.MCP2221()
    buf = [0] * 4

    buf[0] = 0b1000  # GP0 as input
    buf[1] = 0b1000  # GP1 as input
    buf[2] = 0b1000  # GP2 as input
    buf[3] = 0b1000  # GP3 as input

    mcp2221.WriteFlash(MCP2221.FLASH.GP_SETTING, buf)
    buf = mcp2221.ReadFlash(MCP2221.FLASH.GP_SETTING)

    gp0 = buf[0] & 0b1111
    gp1 = buf[1] & 0b1111
    gp2 = buf[2] & 0b1111
    gp3 = buf[3] & 0b1111

    assert (gp0, gp1, gp2, gp3) == (0b1000, 0b1000, 0b1000, 0b1000)


def testReadManufacturer():
    mcp2221 = MCP2221.MCP2221()
    buf = mcp2221.ReadFlash(MCP2221.FLASH.USB_MANUFACTURER)

    valid = (buf[1] << 8 | buf[0]) == ord('M')
    buf_len = len(buf)

    assert (buf_len, valid) == (50, True)


def testInvalidReadFlashAddress():
    mcp2221 = MCP2221.MCP2221()

    with pytest.raises(TypeError):
        mcp2221.ReadFlash(123)


def testInvalidWriteFlashAddress():
    mcp2221 = MCP2221.MCP2221()

    with pytest.raises(TypeError):
        mcp2221.WriteFlash(123, [])


def testInvalidWriteFlashAddressMax():
    mcp2221 = MCP2221.MCP2221()
    buf = [0] * 61

    with pytest.raises(ValueError):
        mcp2221.WriteFlash(MCP2221.FLASH.USB_SERIAL_NUMBER, buf)


def testInvalidWriteFlashAddressMin():
    mcp2221 = MCP2221.MCP2221()

    with pytest.raises(ValueError):
        mcp2221.WriteFlash(MCP2221.FLASH.USB_SERIAL_NUMBER, [])



#!/usr/bin/env python3

import pytest
from MCP2221 import MCP2221


@pytest.mark.parametrize("pin", [2, 3])
def testInitDAC(pin):
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(pin, MCP2221.TYPE.DAC)

    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    gp_type = buf[pin + 22] & 0b111

    assert gp_type == 0b011


def testVrefVDD():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.SetDACVoltageReference(MCP2221.VRM.VDD)

    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    vref = (buf[6] >> 6) & 0b11
    vrm = (buf[6] >> 5) & 0b1

    assert (vref, vrm) == (MCP2221.VRM.VDD.value, 0)


def testVrefVRM():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.SetDACVoltageReference(MCP2221.VRM.REF_2_048V)

    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    vref = (buf[6] >> 6) & 0b11
    vrm = (buf[6] >> 5) & 0b1

    assert (vref, vrm) == (MCP2221.VRM.REF_2_048V.value, 1)


def testInvalidVref():
    mcp2221 = MCP2221.MCP2221()

    with pytest.raises(TypeError):
        mcp2221.SetDACVoltageReference(123)


@pytest.mark.parametrize("pin", [2, 3])
def testWriteDAC(pin):
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(pin, MCP2221.TYPE.DAC)

    mcp2221.WriteDAC(12)

    buf = [0] * 65
    buf[1] = 0x61  # get SRAM settings
    buf = mcp2221._send(buf)

    assert buf[6] & 0b1111 == 12


def testInvalidMaxDAC():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(2, MCP2221.TYPE.DAC)

    with pytest.raises(ValueError):
        mcp2221.WriteDAC(32)


def testInvalidMinDAC():
    mcp2221 = MCP2221.MCP2221()
    mcp2221.InitGP(2, MCP2221.TYPE.DAC)

    with pytest.raises(ValueError):
        mcp2221.WriteDAC(-1)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mcp2221",
    version='v1.0.2',
    author="Pavel Slama",
    author_email="info@pavelslama.cz",
    description="Python driver for MCP2221A",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=['MCP2221', 'MCP2221A', 'GPIO', 'ADC', 'DAC', 'I2C'],
    url="https://github.com/pilotak/python-mcp2221",
    project_urls={
        "Bug Tracker": "https://github.com/pilotak/python-mcp2221/issues",
    },
    packages=setuptools.find_packages(),
    install_requires=['hidapi'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Communications",
        "Topic :: System :: Hardware :: Hardware Drivers",
        "Topic :: System :: Hardware :: Universal Serial Bus (USB)"
    ],
    python_requires='>=3.7',
)



import hid
from time import sleep
from enum import Enum, unique, auto
from typing import Dict, List, Union


class TYPE(Enum):
    INPUT = auto()
    OUTPUT = auto()
    ADC = auto()
    DAC = auto()
    CLOCK_OUT = auto()
    INTERRUPT = auto()
    LED_RX = auto()
    LED_TX = auto()
    LED_I2C = auto()
    SSPND = auto()
    USBCFG = auto()


@unique
class VRM(Enum):
    VDD = 0
    REF_1_024V = 1
    REF_2_048V = 2
    REF_4_096V = 3


@unique
class DUTY(Enum):
    CYCLE_0 = 0
    CYCLE_25 = 1
    CYCLE_50 = 2
    CYCLE_75 = 3


@unique
class CLOCK(Enum):
    DIV_375KHZ = 0b111
    DIV_750KHZ = 0b110
    DIV_1_5MHZ = 0b101
    DIV_3MHZ = 0b100
    DIV_6MHZ = 0b011
    DIV_12MHZ = 0b010
    DIV_24MHZ = 0b001


@unique
class FLASH(Enum):
    CHIP_SETTING = 0x00
    GP_SETTING = 0x01
    USB_MANUFACTURER = 0x02
    USB_PRODUCT_DESCRIPTOR = 0x03
    USB_SERIAL_NUMBER = 0x04
    CHIP_SERIAL_NUMBER = 0x05


class MCP2221:
    def __init__(self, VID=0x04D8, PID=0x00DD, dev=0):
        self.mcp2221 = hid.device()
        self.mcp2221.open_path(hid.enumerate(VID, PID)[dev]["path"])
        self.VID = VID
        self.PID = PID

    def _getConfig(self):
        """ Get current config & prepare for set """

        buf = [0] * 65
        buf[1] = 0x61  # get SRAM settings
        rbuf = self._send(buf)
        # print(rbuf)

        buf[0 + 1] = 0x60  # set SRAM settings

        # Clock Output Divider Value
        buf[2 + 1] |= (rbuf[5] & 0b11111)

        # DAC Voltage Reference
        buf[3 + 1] |= rbuf[6] >> 5

        # ADC Voltage Reference
        buf[5 + 1] |= (rbuf[7] >> 2) & 0b111

        # Interrupt detection
        # TODO

        # GP0 Settings
        buf[8 + 1] = rbuf[22]

        # GP1 Settings
        buf[9 + 1] = rbuf[23]

        # GP2 Settings
        buf[10 + 1] = rbuf[24]

        # GP3 Settings
        buf[11 + 1] = rbuf[25]

        return buf

    def _send(self, buffer: List[int]) -> List[int]:
        """ Send buffer """

        self.mcp2221.write(buffer)
        return self.mcp2221.read(65)

    def SetClockOutput(self, duty: DUTY, clock: CLOCK):
        """ Set clock output """

        if not isinstance(duty, DUTY):
            raise TypeError("Invalid duty cycle value")
        if not isinstance(clock, CLOCK):
            raise TypeError("Invalid clock divider value")

        buf = self._getConfig()
        buf[2 + 1] = 0b10000000  # set mode
        buf[2 + 1] |= clock.value
        buf[2 + 1] |= (duty.value << 3)

        self._send(buf)

    def SetDACVoltageReference(self, ref: VRM):
        """ Set DAC voltage reference """

        if not isinstance(ref, VRM):
            raise TypeError("Invalid DAC voltage reference")

        buf = self._getConfig()
        buf[3 + 1] = 0b10000000  # set mode

        if ref != VRM.VDD:
            buf[3 + 1] |= ref.value << 1
            buf[3 + 1] |= 0b1

        self._send(buf)

    def WriteDAC(self, value: int):
        """ Write DAC value (0-31) """

        if not 0 <= value <= 31:
            raise ValueError("Invalid value")

        buf = self._getConfig()
        buf[4 + 1] |= 0b10000000  # set mode
        buf[4 + 1] |= value

        self._send(buf)

    def SetADCVoltageReference(self, ref: VRM):
        """ Set ADC voltage reference """

        if not isinstance(ref, VRM):
            raise TypeError("Invalid ADC voltage reference")

        buf = self._getConfig()
        buf[5 + 1] = 0b10000000  # set mode

        if ref != VRM.VDD:
            buf[5 + 1] |= ref.value << 1
            buf[5 + 1] |= 0b1  # VRM is used

        self._send(buf)

    def SetInterruptDetection(self):
        # TODO
        raise Exception("Not yet implemented")

    def InitGP(self, pin: int, type: TYPE, value: int = 0):
        """ Init GPIO """

        buf = self._getConfig()
        buf[7 + 1] = 0b10000000  # alter GPIO

        if pin == 0:
            buf[8 + 1] = 0

            if type == TYPE.INPUT:
                buf[8 + 1] |= 1 << 3
            elif type == TYPE.OUTPUT:
                buf[8 + 1] |= (value & 1) << 4
            elif type == TYPE.SSPND:
                buf[8 + 1] |= 1
            elif type == TYPE.LED_RX:
                buf[8 + 1] |= 2
            else:
                raise TypeError("Invalid type on pin GP0")

        elif pin == 1:
            buf[9 + 1] = 0

            if type == TYPE.INPUT:
                buf[9 + 1] |= 1 << 3
            elif type == TYPE.OUTPUT:
                buf[9 + 1] |= (value & 1) << 4
            elif type == TYPE.CLOCK_OUT:
                buf[9 + 1] |= 1
            elif type == TYPE.ADC:
                buf[9 + 1] |= 2
            elif type == TYPE.LED_TX:
                buf[9 + 1] |= 3
            elif type == TYPE.INTERRUPT:
                buf[9 + 1] |= 4
            else:
                raise TypeError("Invalid type on pin GP1")

        elif pin == 2:
            buf[10 + 1] = 0

            if type == TYPE.INPUT:
                buf[10 + 1] |= 1 << 3
            elif type == TYPE.OUTPUT:
                buf[10 + 1] |= (value & 1) << 4
            elif type == TYPE.USBCFG:
                buf[10 + 1] |= 1
            elif type == TYPE.ADC:
                buf[10 + 1] |= 2
            elif type == TYPE.DAC:
                buf[10 + 1] |= 3
            else:
                raise TypeError("Invalid type on pin GP2")

        elif pin == 3:
            buf[11 + 1] = 0

            if type == TYPE.INPUT:
                buf[11 + 1] |= 1 << 3
            elif type == TYPE.OUTPUT:
                buf[11 + 1] |= (value & 1) << 4
            elif type == TYPE.LED_I2C:
                buf[11 + 1] |= 1
            elif type == TYPE.ADC:
                buf[11 + 1] |= 2
            elif type == TYPE.DAC:
                buf[11 + 1] |= 3
            else:
                raise TypeError("Invalid type on pin GP3")

        else:
            raise ValueError("Invalid pin number")

        self._send(buf)

    def ReadAllGP(self):
        """ Read GPIOs in bulk (when set as input or output) """

        buf = [0] * 65
        buf[1] = 0x51  # Get GPIO Values
        buf = self._send(buf)

        if buf[0] == 0x51 and buf[1] == 0x00:
            return [buf[2], buf[4], buf[6], buf[8]]
        else:
            return None

    def ReadGP(self, pin: int) -> Union[int, None]:
        """ Read GPIO pin value (when set as input or output) """

        if not 0 <= pin <= 3:
            raise ValueError("Invalid pin number")

        gpio = self.ReadAllGP()

        if gpio:
            return gpio[pin]
        else:
            return None

    def WriteAllGP(self, gp0: Union[int, None], gp1: Union[int, None],
                   gp2: Union[int, None], gp3: Union[int, None]):
        """ Write GPIO output """

        buf = [0] * 65
        buf[0 + 1] = 0x50  # Set GPIO Values

        if gp0 is not None:
            buf[2 + 1] = 1  # Alter GPIO output
            buf[3 + 1] = gp0  # output value

        if gp1 is not None:
            buf[6 + 1] = 1  # Alter GPIO output
            buf[7 + 1] = gp1  # output value

        if gp2 is not None:
            buf[10 + 1] = 1  # Alter GPIO output
            buf[11 + 1] = gp2  # output value

        if gp3 is not None:
            buf[14 + 1] = 1  # Alter GPIO output
            buf[15 + 1] = gp3  # output value

        self._send(buf)

    def WriteGP(self, pin: int, value: int):
        """ Write GPIO output """

        if not 0 <= pin <= 3:
            raise ValueError("Invalid pin number")

        buf = [0] * 65
        buf[0 + 1] = 0x50  # Set GPIO Values
        buf[2 + pin * 4 + 1] = 1  # Alter GPIO output
        buf[3 + pin * 4 + 1] = value & 1  # output value

        self._send(buf)

    def ReadAllADC(self):
        """ Read ADC in bulk """

        buf = [0] * 65
        buf[0 + 1] = 0x10  # Status/Set Parameters

        buf = self._send(buf)

        if buf[0] == 0x10 and buf[1] == 0x00:
            return [buf[50] | (buf[51] << 8),
                    buf[52] | (buf[53] << 8),
                    buf[54] | (buf[55] << 8)]
        else:
            return None

    def ReadADC(self, channel: int) -> Union[int, None]:
        """ Read specific ADC channel """

        if not 1 <= channel <= 3:
            raise ValueError("Invalid channel number")

        adc = self.ReadAllADC()

        if adc:
            return adc[channel - 1]
        else:
            return None

    def GetDeviceInfo(self) -> Dict[str, Union[str, None]]:
        """ Get device information """

        output = dict()
        output["manufacturer"] = self.mcp2221.get_manufacturer_string()
        output["product"] = self.mcp2221.get_product_string()
        output["serial"] = self.mcp2221.get_serial_number_string()

        return output

    def ReadFlash(self, address: FLASH):
        """ Read data from flash """

        if not isinstance(address, FLASH):
            raise TypeError("Invalid flash address")

        buf = [0] * 65
        buf[0 + 1] = 0xB0  # Read Flash Data
        buf[1 + 1] = address.value

        buf = self._send(buf)

        if buf[0] == 0xB0 and buf[1] == 0x00:
            if address == FLASH.GP_SETTING or \
                    address == FLASH.CHIP_SETTING or \
                    address == FLASH.CHIP_SERIAL_NUMBER:
                return buf[4:(4+buf[2])]
            elif buf[3] == 0x03:
                return buf[4:(4+buf[2]-2)]
            else:
                return []
        else:
            return []

    def WriteFlash(self, address: FLASH, data: List[int]) -> Union[int, None]:
        """ Write data to flash """

        if not isinstance(address, FLASH) \
                or address == FLASH.CHIP_SERIAL_NUMBER:
            raise TypeError("Invalid flash address")

        if len(data) == 0 or len(data) > 60:
            raise ValueError("Invalid data length")

        buf = [0] * 3
        buf[0 + 1] = 0xB1  # Write Flash Data
        buf[1 + 1] = address.value

        fill = [0] * (65 - 3 - len(data))

        buf = self._send([*buf, *data, *fill])

        if buf[0] == 0xB1:
            return buf[1]
        else:
            return None

    def Reset(self):
        """ Reset the device """

        buf = [0x00, 0x70, 0xAB, 0xCD, 0xEF]
        fill = [0] * 60
        self.mcp2221.write([*buf, *fill])
        sleep(1)



#!/usr/bin/env python3

import pytest
from MCP2221 import MCP2221


def testReset():
    mcp2221 = MCP2221.MCP2221()

    try:
        mcp2221.Reset()
    except Exception as err:
        pytest.fail(err)
